=== modified file 'bzrlib/tests/tree_implementations/test_tree.py'
--- bzrlib/tests/tree_implementations/test_tree.py	2008-09-08 20:37:16 +0000
+++ bzrlib/tests/tree_implementations/test_tree.py	2008-09-08 20:43:46 +0000
@@ -17,6 +17,7 @@
 from bzrlib import (
     errors,
     conflicts,
+    osutils,
     revisiontree,
     tests,
     workingtree_4,
@@ -238,3 +239,38 @@
         tree.lock_read()
         self.addCleanup(tree.unlock)
         self.assertEqual(expected, list(tree.extras()))
+
+class TestHasId(TestCaseWithTree):
+
+    def test_has_id(self):
+        work_tree = self.make_branch_and_tree('tree')
+        self.build_tree(['tree/file'])
+        work_tree.add('file', 'file-id')
+        tree = self._convert_tree(work_tree)
+        tree.lock_read()
+        self.addCleanup(tree.unlock)
+        self.assertTrue(tree.has_id('file-id'))
+        self.assertFalse(tree.has_id('dir-id'))
+
+    def test___contains__(self):
+        work_tree = self.make_branch_and_tree('tree')
+        self.build_tree(['tree/file'])
+        work_tree.add('file', 'file-id')
+        tree = self._convert_tree(work_tree)
+        tree.lock_read()
+        self.addCleanup(tree.unlock)
+        self.assertTrue('file-id' in tree)
+        self.assertFalse('dir-id' in tree)
+
+
+class TestGetFileSha1(TestCaseWithTree):
+
+    def test_get_file_sha1(self):
+        work_tree = self.make_branch_and_tree('tree')
+        self.build_tree_contents([('tree/file', 'file content')])
+        work_tree.add('file', 'file-id')
+        tree = self._convert_tree(work_tree)
+        tree.lock_read()
+        self.addCleanup(tree.unlock)
+        expected = osutils.sha_strings('file content')
+        self.assertEqual(expected, tree.get_file_sha1('file-id'))

=== modified file 'bzrlib/transform.py'
--- bzrlib/transform.py	2008-09-08 20:38:46 +0000
+++ bzrlib/transform.py	2008-09-08 20:43:46 +0000
@@ -41,6 +41,7 @@
     has_symlinks,
     lexists,
     pathjoin,
+    sha_file,
     splitpath,
     supports_executable,
 )
@@ -1666,7 +1667,16 @@
             return None
 
     def get_file_sha1(self, file_id, path=None, stat_value=None):
-        return self._transform._tree.get_file_sha1(file_id)
+        trans_id = self._transform.trans_id_file_id(file_id)
+        kind = self._transform._new_contents.get(trans_id)
+        if kind is None:
+            return self._transform._tree.get_file_sha1(file_id)
+        if kind == 'file':
+            fileobj = self.get_file(file_id)
+            try:
+                return sha_file(fileobj)
+            finally:
+                fileobj.close()
 
     def is_executable(self, file_id, path=None):
         if file_id is None:

=== modified file 'bzrlib/tree.py'
--- bzrlib/tree.py	2008-09-20 17:20:39 +0000
+++ bzrlib/tree.py	2008-09-20 17:21:07 +0000
@@ -132,7 +132,8 @@
     def has_id(self, file_id):
         return self.inventory.has_id(file_id)
 
-    __contains__ = has_id
+    def __contains__(self, file_id):
+        return self.has_id(file_id)
 
     def has_or_had_id(self, file_id):
         if file_id == self.inventory.root.file_id:

